from colorlog import ColoredFormatter
import logging

levels = ["critical", "error", "warning", "info", "debug"]
logging_levels = [logging.CRITICAL, logging.ERROR, logging.WARNING,
                  logging.INFO, logging.DEBUG]

# Based on https://stackoverflow.com/a/23964880/5107192
LOG_LEVEL = logging.DEBUG

logging.root.setLevel(LOG_LEVEL)

formatter = ColoredFormatter(
    "%(asctime)-15s %(log_color)s%(levelname)-8s [%(name)s] %(message)s",
    datefmt=None,
    reset=True,
    log_colors={
        'DEBUG': 'cyan',
        'INFO': 'green',
        'WARNING': 'yellow',
        'ERROR': 'red',
        'CRITICAL': 'red,bg_white',
    },
    secondary_log_colors={},
    style='%'
)

handler = logging.StreamHandler()
handler.setLevel(LOG_LEVEL)
handler.setFormatter(formatter)

logger = logging.getLogger('dlgsheet')
logger.setLevel(LOG_LEVEL)
logger.addHandler(handler)


def setLoggerLevel(level):

    if(level in levels):
        logger.setLevel(logging_levels[levels.index(level)])
    else:
        logger.warning("Not valid logging level: " +
                       level + ". Ignoring option.")



from optparse import OptionParser
import os
from pathlib import Path

from dlgsheet import config
from dlgsheet.logger import logger, levels, setLoggerLevel
from dlgsheet.gsheet.api import set_credentials
from dlgsheet.download_data import download_table_values, download_all_tables


usage = "usage: %prog [options] tablename"
parser = OptionParser(usage=usage, prog="dlgsheet")
parser.add_option("-l", "--log-level", dest="loglevel",
                  help="set log level. Available options: " + ",".join(levels))
parser.add_option("-c", "--credentials-file", dest="credentials",
                  help="set credentials file name", metavar="FILE")
parser.add_option("-o", "--output-file", dest="output",
                  help="save to output file", metavar="FILE")
parser.add_option("-d", "--output-folder", dest="output_folder",
                  help="save to output folder", metavar="FOLDER")
parser.add_option("-s", "--spreadsheet-id", dest="spreadsheetid",
                  help="set google spreadsheet id to write on")
parser.add_option("-t", "--sheetname", dest="sheetname",
                  help="set google sheetname to write on")
parser.add_option("-k", "--key-index", type="int", dest="key_index",
                  help="set key index to generate object related to it")

parser.add_option("-K", "--use-keys-table", action="store_true",
                  dest="use_keys_table",
                  default=False, help="use keys table in spreadsheet")

parser.add_option("-T", "--keys-table",
                  dest="keys_table",
                  help="keys table name in spreadsheet. Default '_keys'")

parser.add_option("-n", "--tablename-column",
                  dest="tablename_column",
                  help="tablename column in keys table. Default 'tablename'")

parser.add_option("-i", "--key-index-column",
                  dest="key_index_column",
                  help="key index column in keys table. Default 'key_index'")


# Reference https://stackoverflow.com/a/29301200/5107192
def get_comma_separated_args(option, opt, value, parser):
    setattr(parser.values, option.dest, value.split(','))


parser.add_option(
    "-B",
    "--blacklist",
    type="string",
    action='callback',
    callback=get_comma_separated_args,
    dest="blacklist",
    help="list of tables that won't be considered, separated by commas. Default '_keys'")


def validate_index(index):

    if index is not None:
        try:
            index = int(index)
        except Exception as e:
            logger.error(f"Entered index '{index}' is not an integer.")
            exit(-1)
    return index


def main():

    (options, _) = parser.parse_args()

    if options.loglevel is not None:
        loglevel = options.loglevel
    else:
        loglevel = "info"

    setLoggerLevel(loglevel)

    logger.debug(options)

    DEFAULT_FOLDER_NAME = "output"

    if(options.spreadsheetid is not None):
        spreadsheetid = options.spreadsheetid
    else:
        spreadsheetid = config.google["spreadsheetid"]

    if(not spreadsheetid):
        logger.error(
            '''No spreadsheetid defined, either via option --spreadsheet-id or
            environment variable GOOGLE_SPREADSHEET_ID.''')
        exit(-1)

    if(options.credentials is not None):
        credentials = options.credentials
        auth = set_credentials(credentials)
    else:
        credentials = config.google["credentialsfile"]
        logger.warning("No credentials file defined, usign default: " +
                       credentials)
        if(not Path(credentials).exists()):
            logger.error(
                "Default credentials file doesn't exist, skipping. "
                "Download the credentials file and save as 'key.json' "
                "or specify the name by the --credentials-file option."
            )
            exit(-1)
        auth = set_credentials(credentials)

    if(options.sheetname is not None):

        sheetname = options.sheetname
        defaultfilename = os.path.join(
            DEFAULT_FOLDER_NAME, sheetname + ".json")

        if(options.output is not None):
            filename = options.output
        else:
            filename = defaultfilename
            logger.warning(
                "Not file name provided via --output-file. Usign default: " +
                filename)

        logger.info("Downloading data from spreadsheet: " +
                    spreadsheetid + " from '" +
                    sheetname + "' to " + filename)

        key_index = validate_index(options.key_index)
        if key_index is not None:
            logger.info(f"Using key index {key_index} for selected table.")

        download_table_values(sheetname, filename=filename,
                              spreadsheetid=spreadsheetid, credentials=auth,
                              key_index=key_index)

        logger.info("Task finished")
        exit(0)

    if(options.output_folder is not None):
        foldername = options.output_folder
    else:
        foldername = DEFAULT_FOLDER_NAME
        logger.warning(
            "Not folder name provided via --output-folder. Usign default: " +
            foldername)

    logger.info("Downloading data from spreadsheet: " +
                spreadsheetid + " to folder " + foldername)

    blacklist = options.blacklist
    if blacklist is not None:
        logger.info(f"Blacklisting sheets: {blacklist}")

    if(options.use_keys_table):

        logger.info("Using keys table in spreadsheet to set keys to tables")

        download_all_tables(foldername=foldername,
                            blacklist=blacklist,
                            spreadsheetid=spreadsheetid,
                            credentials=auth,
                            use_keys_table=options.use_keys_table,
                            keys_table=options.keys_table,
                            tablename_column=options.tablename_column,
                            key_index_column=options.key_index_column)

    else:

        key_index = validate_index(options.key_index)

        if key_index is not None:
            logger.info(f"Using key index {key_index} for all tables.")

        download_all_tables(foldername=foldername,
                            blacklist=blacklist,
                            spreadsheetid=spreadsheetid,
                            credentials=auth,
                            key_index=key_index)

    logger.info("Task finished")



import os
from dotenv import load_dotenv

load_dotenv()

google = {
    "spreadsheetid": os.getenv("GOOGLE_SPREADSHEET_ID"),
    "credentialsfile": os.getenv("GOOGLE_AUTH_FILENAME", "key.json"),
    "scopes": ['https://www.googleapis.com/auth/spreadsheets']
}

blacklist = ["_keys"]

keys_table = {
    "name": "_keys",
    "columns": {
        "tablename": {
            "name": "tablename",
            "default_index": 0
        },
        "key_index": {
            "name": "key_index",
            "default_index": 1
        }
    }
}



from dlgsheet.logger import logger


def list_to_object(list_values, header):

    return dict(zip(header, list_values))


def nested_list_to_object(nested_list, header=None, key_index=None):

    if header is None:
        header = nested_list[0]
        nested_list = nested_list[1:]

    if key_index is None:
        return [list_to_object(list_values, header)
                for list_values in nested_list]

    if(key_index >= len(header)):
        logger.error(f"Key index {key_index} is out of range.")
        return None

    header.pop(key_index)
    result = {}

    for list_values in nested_list:

        key = list_values.pop(key_index)
        result[key] = list_to_object(list_values, header)

    return result



import json
import os
from pathlib import Path


def save_to_json_file(data, filename):
    """Save to JSON file

    :data: data from scrapersave
    :filename: filename to save in
    :returns: None

    """
    directory = os.path.dirname(filename)
    Path(directory).mkdir(parents=True, exist_ok=True)

    with open(filename, 'w', encoding='utf8') as json_file:
        json.dump(data, json_file, indent=4, ensure_ascii=False)
    return



from pathlib import Path

from dlgsheet.gsheet.api import get_tables, get_table_values
from dlgsheet.utils.to_object import nested_list_to_object
from dlgsheet.utils.list import find_index
from dlgsheet.logger import logger
from dlgsheet.save_data import save_to_json_file
from dlgsheet import config


def download_table_values(sheetname, filename=None, spreadsheetid=None,
                          credentials=None, key_index=None):

    sheets = get_tables(spreadsheetid=spreadsheetid, credentials=credentials)

    if (sheetname not in sheets):
        logger.error("sheet '" + sheetname +
                     "' not in spreadsheet.")
        return exit(-1)

    data = get_table_values(sheetname, spreadsheetid=spreadsheetid,
                            credentials=credentials)

    l_obj = nested_list_to_object(data["values"], key_index=key_index)

    if(l_obj is None):
        logger.error("Cannot save data of specified table, check your "
                     "configuration.")
    if (filename is None):
        filename = sheetname + ".json"

    save_to_json_file(l_obj, filename)


def get_keys_table_info(sheets, keys_table=None,
                        tablename_column=None,
                        key_index_column=None,
                        spreadsheetid=None,
                        credentials=None):

    keys_table_info = config.keys_table
    if keys_table is not None:
        keys_table_info["name"] = keys_table
    else:
        logger.warning("Not keys table specified, using default "
                       + keys_table_info["name"])

    if keys_table_info["name"] not in sheets:
        logger.error(
            f"Specified keys table '{keys_table_info['name']}' not found in spreadsheet")
        return None
    else:
        tablename_column_info = keys_table_info["columns"]["tablename"]
        key_index_column_info = keys_table_info["columns"]["key_index"]
        if tablename_column is not None:
            tablename_column_info["name"] = tablename_column

        if key_index_column is not None:
            key_index_column_info["name"] = key_index_column

        keys_data = get_table_values(
            keys_table_info["name"],
            spreadsheetid=spreadsheetid,
            credentials=credentials)["values"]

        header = keys_data.pop(0)

        tablename_column_info["index"] = find_index(
            header,
            tablename_column_info["name"],
            tablename_column_info["default_index"])

        key_index_column_info["index"] = find_index(
            header,
            key_index_column_info["name"],
            key_index_column_info["default_index"])

        keys_data = list(zip(*keys_data))

        if tablename_column_info["index"] >= len(keys_data):
            logger.error(f"Index {tablename_column_info['index']} for "
                         "tablename is out of range ")
            return None

        if key_index_column_info["index"] >= len(keys_data):
            logger.error(f"Index {key_index_column_info['index']} for "
                         "key_index is out of range ")
            return None

        tablename_column_info["data"] = keys_data[
            tablename_column_info["index"]]

        key_index_column_info["data"] = keys_data[
            key_index_column_info["index"]]

        keys_table_info["columns"]["tablename"] = tablename_column_info
        keys_table_info["columns"]["key_index"] = key_index_column_info

        logger.debug(f"Keys table info: {keys_table_info}")
        return keys_table_info


def download_all_tables(
        foldername,
        blacklist=None,
        spreadsheetid=None,
        credentials=None,
        key_index=None,
        use_keys_table=False,
        keys_table=None,
        tablename_column=None,
        key_index_column=None):

    folder = Path(foldername)

    if blacklist is None:
        blacklist = config.blacklist

    sheets = get_tables(spreadsheetid=spreadsheetid, credentials=credentials)

    keys_table_info = None
    if use_keys_table:
        keys_table_info = get_keys_table_info(
            sheets,
            keys_table=keys_table,
            tablename_column=tablename_column,
            key_index_column=key_index_column,
            spreadsheetid=spreadsheetid,
            credentials=credentials
        )

        if keys_table_info is None:
            logger.error(f"Cannot use specified table "
                         "as table info, please check the configuration, "
                         "or use the default mode with no keys.")
            exit(-1)

    for sheetname in sheets:

        if sheetname in blacklist:
            continue

        filename = sheetname + ".json"
        save_filename = folder / filename

        logger.info(
            "Downloading table '" +
            sheetname +
            "' to " +
            str(save_filename))

        data = get_table_values(sheetname, spreadsheetid=spreadsheetid,
                                credentials=credentials)

        if use_keys_table and keys_table_info is not None:
            if sheetname in keys_table_info["columns"]["tablename"]["data"]:
                idx = keys_table_info["columns"]["tablename"]["data"].index(
                    sheetname)
                key_index = keys_table_info["columns"]["key_index"]["data"][idx]
                logger.info(f"Using index {key_index} for table '{sheetname}'")

        l_obj = nested_list_to_object(data["values"], key_index=key_index)

        if(l_obj is None):
            logger.error("Cannot save data of specified table, check your "
                         "configuration.")
            continue

        save_to_json_file(l_obj, save_filename)


if __name__ == "__main__":
    download_all_tables("output")



from dlgsheet import config
from googleapiclient.discovery import build
from google.oauth2 import service_account
from pathlib import Path

if(Path(config.google["credentialsfile"]).exists()):
    globalcredentials = service_account.Credentials.from_service_account_file(
        config.google["credentialsfile"], scopes=config.google["scopes"])


def set_credentials(filename):
    credentials = service_account.Credentials.from_service_account_file(
        filename, scopes=config.google["scopes"])
    return credentials


def get_spreadsheet_service(spreadsheetid=None,
                          credentials=None):

    if(credentials is None):
        credentials = globalcredentials

    if(spreadsheetid is None):
        spreadsheetid = config.google["spreadsheetid"]

    service = build('sheets', 'v4', credentials=credentials)

    sheet = service.spreadsheets()
    return sheet, spreadsheetid


def get_tables(spreadsheetid=None,
              credentials=None):

    sheet, spreadsheetid = get_spreadsheet_service(spreadsheetid=spreadsheetid,
                                                 credentials=credentials)

    result = sheet.get(spreadsheetId=spreadsheetid).execute()
    sheetnames = [sheet["properties"]["title"] for sheet in result["sheets"]]
    return sheetnames


def get_table_values(sheetname, spreadsheetid=None,
                credentials=None):

    sheet, spreadsheetid = get_spreadsheet_service(spreadsheetid=spreadsheetid,
                                                 credentials=credentials)

    result = sheet.values().get(spreadsheetId=spreadsheetid,
                              valueRenderOption="UNFORMATTED_VALUE",
                              dateTimeRenderOption="FORMATTED_STRING",
                              range=f"{sheetname}").execute()

    return result


if __name__ == '__main__':

    result = get_tables()
    print(result)
    values = get_table_values("skills")
    print(values)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="dlgsheet",
    version="0.0.4",
    author="Luighi Viton-Zorrilla",
    author_email="luighiavz@gmail.com",
    description="Script to Google Sheets to JSON files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/LuighiV/download-gsheet",
    project_urls={
        "Bug Tracker": "https://github.com/openpolitica/download-gsheet/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License ",
        "Operating System :: POSIX :: Linux ",
    ],
    packages=setuptools.find_packages(
        include=[
            'dlgsheet',
            'dlgsheet.*']),
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=[
        'colorlog',
        'google_api_python_client',
        'python-dotenv'],
    entry_points={
        'console_scripts': [
            'dlgsheet=dlgsheet.cli:main',
        ],
    },
)
